@@ -45,18 +45,18 @@ def idtosparse(df, G1=None, G2=None):
     sp_mtx = csr_matrix((df.value, (df.x_id, df.y_id)), shape=(G1, G2))
     return sp_mtx
 
-def csntoflat(csn):
-    if type(csn) is list:
-        n2 = len(csn)
-        n1 = csn[0].shape[0]
+def csntoflat(csn_mat):
+    if type(csn_mat) is list:
+        n2 = len(csn_mat)
+        n1 = csn_mat[0].shape[0]
         csn_flat = np.zeros((int(n1*(n1-1)/2), n2))
         k = 0
         for i in range(0, n1-1):
             for j in range(i+1, n1):
-                csn_flat[k, :] = np.asarray([item[i,j] for item in csn_loc_mat])
+                csn_flat[k, :] = np.asarray([item[i,j] for item in csn_mat])
                 k = k + 1
     else: 
-        (n1, n11, n2) = csn.shape
+        (n1, n11, n2) = csn_mat.shape
         if n1 != n11: 
             print('dimension not match!')
             return
@@ -578,6 +578,7 @@ def csn_loc(data_full, knn_index, wd_q = 0.1, dev = True, md = 1, iteration = Fa
     return csn_mat_list # csn_mat
     
 def csn_rec_loc(data1, data2, knn_index, wd_q = 0.1, dev = True, md = 1, iteration = False, ncore = 4):
+    eps = np.finfo(float).eps
     (G1, N) = data1.shape
     G2 = data2.shape[0]
     (nk, nc) = knn_index.shape
@@ -636,6 +637,7 @@ def csn_rec_loc(data1, data2, knn_index, wd_q = 0.1, dev = True, md = 1, iterati
 def csn_block_loc(data, knn_index, M = 100, wd_q = 0.1, dev = True, md = 1, iteration = False, ncore = 4):
     (G, K) = data.shape
     n = math.ceil(G/M)
+    eps = np.finfo(float).eps
     group_n = np.zeros(G)
     for i in range(0, n):
         group_n[i*M:min((i+1)*M, G)] = i
@@ -661,33 +663,23 @@ def csn_block_loc(data, knn_index, M = 100, wd_q = 0.1, dev = True, md = 1, iter
     csn_mtx = [idtosparse(item, G, G) for item in csn_mtx_id]
     return csn_mtx
    
-def create_D_mat(csn_mat, ncore = 4):
+def create_D_mat(csn_mat):
     if type(csn_mat) is list:
         T = len(csn_mat)
     else:
         T = csn_mat.shape[2]
+    
     D_mat = np.zeros((T, T))
-    s_mtx = np.ones((T, T)) - np.tri(T)
-    (I, J, S) = find(s_mtx)
-    L = len(I)
+    
     if type(csn_mat) is list:
-        def inner_fun(m):
-            i = I[m]
-            j = J[m]
-            a = sum(abs(csn_mat[i]-csn_mat[j])).max()
-            return a
-        A = np.asarray(Parallel(n_jobs = ncore)(delayed(inner_fun)(m) for m in range(0, L)))
-        D_mat[I, J] = A
-        D_mat[J, I] = A
+        for i in range(0, T-1):
+            for j in range(i+1, T):
+                D_mat[i, j] = np.linalg.norm((csn_mat[i] - csn_mat[j]).todense(), ord = 1)
     else:
-        def inner_fun(m):
-            i = I[m]
-            j = J[m]
-            a = max(sum(abs(csn_mat[:, :, i] - csn_mat[:, :, j])))
-            return a
-        A = np.asarray(Parallel(n_jobs = ncore)(delayed(inner_fun)(m) for m in range(0, L)))
-        D_mat[I, J] = A
-        D_mat[J, I] = A
+        for i in range(0, T-1):
+            for j in range(i+1, T):
+                D_mat[i, j] = np.linalg.norm(csn_mat[:, :, i] - csn_mat[:, :, j], ord = 1)
+    D_mat = D_mat + D_mat.transpose()
     return D_mat
     
 def distance_test(D_mat, change, num_perm = 1000):
